"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = undefined;

var _koaRouter = require("koa-router");

var _koaRouter2 = _interopRequireDefault(_koaRouter);

var _configYaml = require("config-yaml");

var _configYaml2 = _interopRequireDefault(_configYaml);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let RouterBuilder = class RouterBuilder {

    constructor(container, filePath) {
        this.container = container;
        if (filePath) {
            this.filePath = filePath;
        }
    }

    async populateRoutes(router, config) {
        for (let routeId in config) {
            let route = config[routeId];

            if (!route.path) {
                throw new Error(`Path is required for the route "${routeId}"`);
            }
            if (!route.controller) {
                throw new Error(`Controller is required for the route "${routeId}"`);
            }

            let path = route.path;
            let controller = await this.container.get(route.controller.substring(1));
            let actionName = route.action;
            let action = controller[actionName];

            router.get(routeId, path, action);
        }
    }

    async build() {
        let router = new _koaRouter2.default();

        if (this.filePath) {
            const config = (0, _configYaml2.default)(this.filePath);
            await this.populateRoutes(router, config);
        }

        return [router.routes(), router.allowedMethods()];
    }
};
exports.default = RouterBuilder;
module.exports = exports["default"];